from ._version import grid_version

_base_url = f"https://cdn.jsdelivr.net/npm/ag-grid-community@{grid_version}/styles"

BASE = f"{_base_url}/ag-grid.min.css"
ALPINE = f"{_base_url}/ag-theme-alpine.min.css"
BALHAM = f"{_base_url}/ag-theme-balham.min.css"
MATERIAL = f"{_base_url}/ag-theme-material.min.css"
QUARTZ = f"{_base_url}/ag-theme-quartz.min.css"
